program PrjRender;

{$APPTYPE CONSOLE}

uses
  System.SysUtils,
  MVCFramework.Logger,
  MVCFramework.Commons,
  MVCFramework.REPLCommandsHandlerU,
  Web.ReqMulti,
  Web.WebReq,
  Web.WebBroker,
  IdContext,
  IdHTTPWebBrokerBridge,
  uController.Render in 'uController.Render.pas',
  uWebModule.Render in 'uWebModule.Render.pas' {WebModuleRender: TWebModule},
  dmConexion.Render in 'dmConexion.Render.pas' {dmConexion: TDataModule},
  uModel.Render in 'uModel.Render.pas';

{$R *.res}


procedure RunServer(APort: Integer);
var
  LServer: TIdHTTPWebBrokerBridge;
  LCustomHandler: TMVCCustomREPLCommandsHandler;
  LCmd: string;
begin
  Writeln('** DMVCFramework Server ** build ' + DMVCFRAMEWORK_VERSION);
  LCmd := 'start';
  if ParamCount >= 1 then
    LCmd := ParamStr(1);

  LCustomHandler := function(const Value: String; const Server: TIdHTTPWebBrokerBridge; out Handled: Boolean): THandleCommandResult
    begin
      Handled := False;
      Result := THandleCommandResult.Unknown;

      // Write here your custom command for the REPL using the following form...
      // ***
      Handled := False;
      if (Value = 'help') then begin
        REPLEmit('Ayuda sobre comandos del servidor: ' + sLineBreak +
                 '   start:          Pausa el servidor.' + sLineBreak +
                 '   stop:           Arranca de nuevo el servidor.' + sLineBreak +
                 '   exit/quit:      Finaliza el servidor.' + sLineBreak +
                 '   -------------------------------------' + sLineBreak +
                 '   help:           Mostrar esta pantalla de ayuda.' + sLineBreak +
                 '   apiversion:     Mostrar la versin de la API.' + sLineBreak +
                 '   info:           Mostrar informacin del servidor.' + sLineBreak +
                 '   datetime:       Fecha y hora del servidor.');
        Result := THandleCommandResult.Continue;
        Handled := True;
      end
      else if (Value = 'apiversion') then begin
        REPLEmit('1.0.0 (beta)');
        Result := THandleCommandResult.Continue;
        Handled := True;
      end
      else if (Value = 'info') then begin
        REPLEmit('Servidor desarrollado con DMVCFramework, ' + DMVCFRAMEWORK_VERSION);
        Result := THandleCommandResult.Continue;
        Handled := True;
      end
      else if (Value = 'datetime') then begin
        REPLEmit(DateTimeToStr(Now));
        Result := THandleCommandResult.Continue;
        Handled := True;
      end;
    end;

  LServer := TIdHTTPWebBrokerBridge.Create(nil);
  try
    LServer.OnParseAuthentication := TMVCParseAuthentication.OnParseAuthentication;
    LServer.DefaultPort := APort;

    { more info about MaxConnections
      http://www.indyproject.org/docsite/html/frames.html?frmname=topic&frmfile=TIdCustomTCPServer_MaxConnections.html }
    LServer.MaxConnections := 0;

    { more info about ListenQueue
      http://www.indyproject.org/docsite/html/frames.html?frmname=topic&frmfile=TIdCustomTCPServer_ListenQueue.html }
    LServer.ListenQueue := 200;
    {required if you use JWT middleware }
    LServer.OnParseAuthentication := TMVCParseAuthentication.OnParseAuthentication;

    WriteLn('Write "quit" or "exit" to shutdown the server');
    repeat
      if LCmd.IsEmpty then
      begin
        Write('-> ');
        ReadLn(LCmd)
      end;
      try
        case HandleCommand(LCmd.ToLower, LServer, LCustomHandler) of
          THandleCommandResult.Continue:
            begin
              Continue;
            end;
          THandleCommandResult.Break:
            begin
              Break;
            end;
          THandleCommandResult.Unknown:
            begin
              REPLEmit('Unknown command: ' + LCmd);
            end;
        end;
      finally
        LCmd := '';
      end;
    until False;

  finally
    LServer.Free;
  end;
end;

begin
  ReportMemoryLeaksOnShutdown := True;
  IsMultiThread := True;
  try
    if WebRequestHandler <> nil then
      WebRequestHandler.WebModuleClass := WebModuleClass;
    WebRequestHandlerProc.MaxConnections := 1024;
    RunServer(8080);
  except
    on E: Exception do
      Writeln(E.ClassName, ': ', E.Message);
  end;
end.
