unit uController.Render;

interface

uses
  MVCFramework, MVCFramework.Commons, MVCFramework.Serializer.Commons;

type
  EMVCRenderException = class(EMVCException);
  EMVCDatabaseException = class(EMVCException);

  [MVCPath('/api')]
  TControllerRender = class(TMVCController)
  private
    procedure RenderAsText;
    procedure RenderAsHtml;
    // [MVCProduces('text/html')]
    // procedure RenderAsHtml;
    procedure RenderAsJSON;
    procedure RenderAsDataset;
    procedure RenderObject;
    procedure RenderException;

  public
    [MVCPath]
    [MVCHTTPMethod([httpGET])]
    procedure Index;

    [MVCPath('/render')]
    [MVCHTTPMethod([httpGET])]
    [MVCProduces('image/png')]
    procedure RenderAsStream;



  public
  end;

implementation

uses
  System.SysUtils, MVCFramework.Logger, System.StrUtils, System.Classes,
  System.JSON, System.JSON.Types, System.Generics.Collections,
  dmConexion.Render, FireDAC.Comp.Client,
  uModel.Render;

procedure TControllerRender.Index;
begin
  //use Context property to access to the HTTP request and response 
  Render('Hello DelphiMVCFramework World');
end;


procedure TControllerRender.RenderAsDataset;
var
  dm:TdmConexion;
begin
  dm := TdmConexion.Create(nil);
  dm.qGeneric.Open('SELECT * FROM TRACKS LIMIT 3');
  Render(dm.qGeneric);
end;

procedure TControllerRender.RenderAsHtml;
begin
  // ContentType := 'text/html';
  Render('<h2>El servidor est funcionando.</h2><br>' +
         'A la espera de peticiones.');
end;

procedure TControllerRender.RenderAsJSON;
var
  arr:TJSONArray;
begin
  // Render(TJSONObject.Create(TJSONPair.Create('result', '200')));

  arr := TJSONArray.Create;
  arr.Add(TJSONObject.Create(TJSONPair.Create('1', 'Oro')));
  arr.Add(TJSONObject.Create(TJSONPair.Create('2', 'Plata')));
  arr.Add(TJSONObject.Create(TJSONPair.Create('3', 'Bronze')));
  Render(arr);
end;

procedure TControllerRender.RenderAsStream;
var
  f:TFileStream;
begin
  f := TFileStream.Create('c:\dmvc\Ejemplos\Render2\avatar.png', fmOpenRead or fmShareDenyWrite);
  Render(f, False);
end;

procedure TControllerRender.RenderAsText;
begin
  Render('El servidor est funcionando.');
end;

procedure TControllerRender.RenderException;
begin
//  raise EMVCDatabaseException.Create('Error de conexin a la Base de Datos');
  raise EMVCDatabaseException.Create('Error de Base de Datos', 'La conexion no se ha podido realizar', 33, 401);
end;

procedure TControllerRender.RenderObject;
var
  usu1, usu2:TUsuario;
  lusu:TObjectList<TUsuario>;
begin
  usu1 := TUsuario.Create;
  usu1.Id := 1;
  usu1.Nombre := 'Germn';
  usu1.Apellido := 'Estvez';
  usu1.Email := 'german_ral@hotmail.com';

  usu2 := TUsuario.Create;
  usu2.Id := 2;
  usu2.Nombre := 'Mara';
  usu2.Apellido := 'Lopez';
  usu2.Email := 'mlopez@hotmail.com';

  lusu := TObjectList<TUsuario>.CReate;
  lusu.Add(usu1);
  lusu.Add(usu2);

  Render<TUsuario>(lusu);


//  Render(usu);


end;

end.
